package net.sf.corn.cps;

import java.util.Collections;
import java.util.Set;

import java.util.HashSet;
import java.util.regex.Pattern;

/**
 * This filter is designed to filter class names to use with the {@link CPScanner}  
 * @author serhat.dirik
 */
public class ClassNameFilter implements CPResourceFilter {
	private Set<Pattern> classNamePatterns = new HashSet<Pattern>();
	public ClassNameFilter() {
	}
	
	/**
	 * Constructs a class name filter with the given class name. The start ('*') character 
	 * can be used with this filter to specify resource name 
	 * @param name
	 */
	public ClassNameFilter(String name) {
		append(name);
	}
	/**
	 * Appends another resource name to this filter
	 * @param name the resource name need to be filtered, the star ('*') character can be used
	 * with the name to represent unlimited number of any character and the underscore character ('_') 
	 * can be used to represent single any character.
	 * @return this
	 */
	public ClassNameFilter append(String name){
		String ptrn = TextHelper.prepName(name);
		Pattern pattern = Pattern.compile(ptrn);
		classNamePatterns.add(pattern);
		return this;
	}
	
	/**
	 * Gives list of class names that defined in this filter
	 * @return an unmodifiable collection of package names
	 */
	public Set<Pattern> getClassNamePatterns() {
		return Collections.unmodifiableSet(classNamePatterns);
	}

 
	public boolean accept(Object subject) {
        if(classNamePatterns.size() >0 && filterable(subject)){
        	RootedURL rootedUrl = (RootedURL) subject;
        	String className = rootedUrl.getResourceName();
        	//Replace $ for inner classes
        	className = className.replace('$', '.');
        	if(className.indexOf(".class") >0){
        		className = className.substring(0,className.indexOf(".class"));
        	}
        	
        	for (Pattern name : classNamePatterns) {
				 if( name.matcher(className).matches() ) return true;
			}
        	return false;
        }else{
        	return true;	
        }
	}
 
	public boolean filterable(Object subject) {
		return (subject != null && subject instanceof RootedURL);
	}
	
	@Override
	public String toString() {
		return classNamePatterns.toString();
	}
}
