package net.sf.corn.cps;

import java.util.Collections;
import java.util.Set;

import java.util.HashSet;
import java.util.regex.Pattern;

/**
 * This filter is designed to filter package names to use with the {@link CPScanner}  
 * @author serhat.dirik
 */
public class PackageNameFilter implements CPResourceFilter {
	private Set<Pattern> pnamePatterns = new HashSet<Pattern>();
	public PackageNameFilter() {
	}
	
	/**
	 * Constructs a package name filter with the given package name. The star ('*') character 
	 * can be used with this filter to specify package names 
	 * @param name
	 */
	public PackageNameFilter(String name) {
		append(name);
	}
	/**
	 * Appends another package name to this filter
	 * @param name the package name need to be filtered, , the star ('*') character can be used
	 * with the name to represent unlimited number of any character and the underscore character ('_') 
	 * can be used to represent single any character.
	 * @return this
	 */
	public PackageNameFilter append(String name){
		String ptrn = TextHelper.prepName(name);
		Pattern pattern = Pattern.compile(ptrn);
		pnamePatterns.add(pattern);
		return this;
	}
	
	/**
	 * Gives list of package names that defined in this filter
	 * @return an unmodifiable collection of package names
	 */
	public Set<Pattern> getPackageNamePatterns() {
		return Collections.unmodifiableSet(pnamePatterns);
	}

 
	public boolean accept(Object subject) {
        if(pnamePatterns.size() >0 && filterable(subject)){
        	RootedURL rootedUrl = (RootedURL) subject;
        	String packageName = rootedUrl.getPackageName();
        	for (Pattern name : pnamePatterns) {
				 if( name.matcher(packageName).matches() ||
					 name.matcher(packageName.replace('/', '.')).matches()	 ) {
					 return true;
				 }
			}
        	return false;
        }else{
        	return true;	
        }
	}
 
	public boolean filterable(Object subject) {
		return (subject != null && subject instanceof RootedURL);
	}
	
	@Override
	public String toString() {
		return pnamePatterns.toString();
	}
}
