package net.sf.corn.cps;

import java.util.HashSet;
import java.util.Set;

/**
 * Reject filter is a bag of filters that rejects resources those accepted by contained filters. 
 * This filter can join contained filters with logical 'or' (by default) or logical 'and' operator 
 * @author Serhat Dirik
 *
 */
public class RejectFilter implements CPResourceFilter {

	private boolean combineWithOr = true;
	private Set<CPResourceFilter> filters = new HashSet<CPResourceFilter>();
	
 
	public boolean accept(Object subject) {
		int vote=0;
		for(CPResourceFilter filter:filters){
			if(! filter.filterable(subject)) continue;
			if(combineWithOr){
				if (filter.accept(subject)) return false;
			}else{
				if(filter.accept(subject)) {
					vote++;
				}
			}	
		}
		
		if(!combineWithOr){
			return vote != filters.size();
		}else{
			return true;
		}
	}

 
	public boolean filterable(Object subject) {
		for(CPResourceFilter filter:filters){
			if(filter.filterable(subject)){
				return true;
			}
		}
		return false;
	}

	/*
	 * Builders
	 */
	/**
	 * Combines the given filters with the logical 'or' operator.
	 * This is the default behavior of this filter
	 * @return this
	 */
	public RejectFilter combineWithOr(){
		combineWithOr = true;
		return this;
	}
	/**
	 * Combines the given filters with the logical 'and' operator
	 * @return this
	 */
	public RejectFilter combineWithAnd(){
		combineWithOr = false;
		return this;
	}
	
	/**
	 * Appends another filter to this filter
	 * @param filter that need to be combined within this 
	 * @return this
	 */
	public RejectFilter appendFilter(CPResourceFilter  filter){
		filters.add(filter);
		return this;
	}
	
	
	@Override
	public String toString() {
		return (combineWithOr ? "OR:" : "AND:")+filters.toString();
	}
}
