package net.sf.corn.cps;

import java.util.Collections;
import java.util.Set;

import java.util.HashSet;
import java.util.regex.Pattern;

/**
 * This filter is designed to filter resource names to use with the {@link CPScanner}  
 * @author serhat.dirik
 */
public class ResourceNameFilter implements CPResourceFilter {
	private Set<Pattern> resNamePatterns = new HashSet<Pattern>();
	public ResourceNameFilter() {
	}
	
	/**
	 * Constructs a resource name filter with the given resource name. The start ('*') character 
	 * can be used with this filter to specify resource name 
	 * @param name
	 */
	public ResourceNameFilter(String name) {
		append(name);
	}
	/**
	 * Appends another resource name to this filter
	 * @param name the resource name need to be filtered, , the star ('*') character can be used
	 * with the name to represent unlimited number of any character and the underscore character ('_') 
	 * can be used to represent single any character.
	 * @return this
	 */
	public ResourceNameFilter append(String name){
		String ptrn = TextHelper.prepName(name);
		Pattern pattern = Pattern.compile(ptrn);
		resNamePatterns.add(pattern);
		return this;
	}
	
	/**
	 * Gives list of resource names that defined in this filter
	 * @return an unmodifiable collection of package names
	 */
	public Set<Pattern> getResourceNamePatterns() {
		return Collections.unmodifiableSet(resNamePatterns);
	}

 
	public boolean accept(Object subject) {
        if(resNamePatterns.size() >0 && filterable(subject)){
        	RootedURL rootedUrl = (RootedURL) subject;
        	String resourceName = rootedUrl.getResourceName();
        	for (Pattern name : resNamePatterns) {
				 if( name.matcher(resourceName).matches() ) return true;
			}
        	return false;
        }else{
        	return true;	
        }
	}
 
	public boolean filterable(Object subject) {
		return (subject != null && subject instanceof RootedURL);
	}
	
	@Override
	public String toString() {
		return resNamePatterns.toString();
	}
}
